/*
 * extract.c
 * This file provides the extract functions required by gnozip
 */

#include "common.h"
#include "exit.h"
#include "tar.h"
#include "zip.h"
#include "gzip.h"
#include "bzip2.h"
#include "extract.h"

int
gz_extract ()
{
  gchar *name;
  gz_list_entry *data;
  gint filelenght = 0;
  
  switch (filetype)
  {
   case FILE_ZIP:
     gz_zip_extract (filename, extract_dir);
     break;
   case FILE_GZIP:
     data = filelist->data;
     name = data->name;
     filelenght = atoi (data->size);
     gz_gzip_extract (filename, filelenght, extract_dir, name);
     break;
   case FILE_BZIP2:
     data = filelist->data;
     name = data->name;
     gz_bzip2_extract (filename, extract_dir, name);
     break;
   case FILE_TGZ:
     gz_tar_extract (filename, extract_dir, TAR_GZ);
     break;
   case FILE_TBZ2:
     gz_tar_extract (filename, extract_dir, TAR_BZ2);
     break;
   case FILE_TAR:
     gz_tar_extract (filename, extract_dir, TAR_TAR);
     break;
   case FILE_ARJ:
     break;
   case FILE_LHA:
     break;
   case FILE_RAR:
     break;
   default:
     ERROR ("gz_extract", _("no files open"))
  }

  return 0;
}

