/*
 * tar.c
 * This file provides the tar, tar.gz, tar.bz2 file operations interface
 */

#include "common.h"
#include "tar.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

extern gz_config config;

GList *
gz_tar_list (const gchar *filename, int compression)
{
	GList *list = NULL;
	char  *command;
	FILE  *input = NULL;
	gz_list_entry *entry;
	char topt [5];
	gchar *barwname = "appbar1";
	/* gint innum = 0; */
	GtkWidget *mainappbar;
  
	mainappbar = lookup_widget (main_win, barwname);
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);
 
	switch (compression)
	{
	case 0:
		sprintf (topt, "tvf");
		break;
	case 1:
		sprintf (topt, "tvzf");
		break;
	case 2:
		sprintf (topt, "tvIf");
		break;
	default:
		FATAL_ERROR ("gz_tar_list:",_("Wrong Parameter"))
	}
	command = g_strdup_printf ("%s %s %s", config.progs.tar, topt, filename);
	DEBUG ("tar command:", command)
	input = popen (command, "r");
	if (input == NULL) ERROR ("gz_tar_list", _("cannot execute tar command"))

	/*
	 * Redirezionare stderror in modo da riportare gli errori del tar ? 
	 * E' necessario poi rimetterlo a posto ? 
	*/
  
	while (TRUE)
	{
		char entrystr [11 +18 +11 +11 +9 +PATH_MAX +NAME_MAX +1];
    
		/* Read a line at a time */
		entrystr[0] = '\0';
		fgets (entrystr, 11 +18 +11 +11 +9 +PATH_MAX +NAME_MAX, input);
		if (ferror (input)) FATAL_ERROR ("gz_tar_list:",strerror (ferror (input)))
		/* Check if the line is not too long */
		if (feof (input)) break;
		if (entrystr[strlen (entrystr) - 1] != '\n')
			FATAL_ERROR ("gz_tar_list:", _("Too many chars in a file entry"))
		else entrystr[strlen (entrystr) - 1] = '\0';
		DEBUG ("gz_tar_list:", entrystr)

		g_strchomp (entrystr);
		entry = gz_tar_entry (entrystr);
		list = g_list_append (list, entry);

	}
	/* gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/????);
	 */
	
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,1);
	pclose (input);
	return list;
}

gz_list_entry *
gz_tar_entry (char * entrystr)
{
	gz_list_entry *entry;
	char *perms, *owner, *size, *date, *time, *name, *path;
	char *pes; /* entry string pointer */
	int len; /* string lenght */

	entry = g_malloc0 (sizeof (gz_list_entry));
   
	pes = strdup (entrystr);
	perms = strsep(&pes, " ");
	if ((len = strlen (perms)) > 10)
	{
		WARNING ("gz_tar_list:", _("perms field too long"))
		len = 10;
	}
	entry->perms = g_malloc0 (len+1);
	strncpy (entry->perms, perms, len);
	/*    DEBUG ("gz_tar_list:", entry->perms); */
    
	g_strchug (pes); /* Get Out spaces */
	owner = strsep (&pes, " ");
	if ((len = strlen (owner)) > 17)
	{
		WARNING ("gz_tar_list:", _("owner field too long"))
		len = 17;
	}
	entry->owner = g_malloc0 (len+1);
	strncpy (entry->owner, owner, len);
	/*    DEBUG ("gz_tar_list:", entry->owner); */
    
	g_strchug (pes); /* Get Out spaces */
	size = strsep(&pes, " ");
	if ((len = strlen (size)) > 10)
	{
		WARNING ("gz_tar_list:", _("size field too long"))
		len = 10;
	}
	entry->size = g_malloc0 (len+1);
	strncpy (entry->size, size, len);
	/*    DEBUG ("gz_tar_list:", entry->size); */
    
	g_strchug (pes); /* Get Out spaces */
	date = strsep(&pes, " ");
	if ((len = strlen (date)) > 10)
	{
		WARNING ("gz_tar_list:", _("date field too long"))
		len = 10;
	}
	entry->date = g_malloc0 (len+1);
	strncpy (entry->date, date, len);
	/*    DEBUG ("gz_tar_list:", entry->date); */
    
	g_strchug (pes); /* Get Out spaces */
	time = strsep(&pes, " ");
	if ((len = strlen (time)) > 8)
	{
		WARNING ("gz_tar_list:", _("time field too long"))
		len = 8;
	}
	entry->time = g_malloc0 (len+1);
	strncpy (entry->time, time, len);
	/*    DEBUG ("gz_tar_list:", entry->time); */
    
	g_strchug (pes); /* Get Out spaces */
	path = pes;
	if ((name = strrchr (path, '/')))
	{
		name++;
		if ((len = (strlen (path) - strlen (name))) > PATH_MAX)
		{
			WARNING ("gz_tar_list:", _("path name too long"))
			len = PATH_MAX;
		}
		entry->path = g_malloc0 (len+1);
		strncpy (entry->path, path, len);
		/*    DEBUG ("gz_tar_list:", entry->path); */
	}
	else name = pes;
	if ((len = strlen (name)) > PATH_MAX +NAME_MAX) 
	{
		WARNING ("gz_tar_list:", _("name field too long"))
		len = PATH_MAX +NAME_MAX;
	}
	if (len !=0 )
	{
		entry->name = g_malloc0 (len+1);
		strncpy (entry->name, name, len);
	}
	else entry->name = g_strdup_printf ("<DIR>");
	/*    DEBUG ("gz_tar_list:", entry->name); */

	return entry;
}
	
int
gz_tar_extract (gchar *filename, gchar *extr_dir, int compression)
{
  char *command;
  char *errstr;
  char topt [5];
  FILE *input = NULL;
  gchar *barwname = "appbar1";
  gint innum = 0;
  GtkWidget *mainappbar;

  mainappbar = lookup_widget (main_win, barwname);
  
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);
  
  switch (compression)
  {
      case 0:
         sprintf (topt, "xvf");
         break;
      case 1:
         sprintf (topt, "xvzf");
         break;
      case 2:
         sprintf (topt, "xvIf");
         break;
      default:
         FATAL_ERROR ("gz_tar_extract:", _("Wrong Parameter"))
  }

  errstr = g_strdup_printf ("Extract %s to: %s\n", filename, extr_dir);
  DEBUG ("gz_tar_extract:", errstr);
  command = g_strdup_printf ("%s %s %s -C %s", config.progs.tar, topt, filename, extr_dir);
  DEBUG ("tar command:", command);
  input = popen (command, "r");
  while (TRUE)
  {
      char entrystr [8192];

      fgets (entrystr, 8192, input);
      if (ferror (input)) FATAL_ERROR ("gz_tar_extract:",strerror (ferror (input)))
          /* Check if the line is not too long */
      if (feof (input)) break;
      DEBUG ("gz_tar_extract:", entrystr);
      /* update prograss bar */
      g_strchomp (entrystr);
      if (entrystr[strlen (entrystr) - 1] != '/')
      {
          gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/flnum);
          while (gtk_events_pending ())
                 gtk_main_iteration ();
      }

  }
  pclose (input);
  return FALSE;
}

int 
gz_tar_add (gchar *filename, GList *add_list, gchar *add_dir, int compression)
{
  char *command = NULL;
  char *entrystr;
  FILE  *input = NULL;
  FILE  *output = NULL;
  gchar barwname[] = "appbar1";
  gchar *lfnam;
  gchar cr = '\n';
  char *tempfile = NULL;
  char *temptar = NULL;
  int tempfd, tarfd;
  int num, i, count;
  gint innum = 0;
  gint totnum = 0;
  GtkWidget *mainappbar;
  struct stat filestat;
  GtkWidget *main_clist = NULL;
  gz_list_entry *entry;

  main_clist = gz_mainclist_get_widget ();
  mainappbar = lookup_widget (main_win, barwname);
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);

  tempfile = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
  if ((tempfd = mkstemp (tempfile)) == -1) 
  {
	FATAL_ERROR ("gz_tar_add:", _("Unable to create temporary file"))
	return -1;
  }
  DEBUG ("gz_tar_add: temp file:", tempfile);
  
  num = g_list_length (add_list);
  for (i = 0; i < num; i++)
  {
	lfnam = g_list_nth_data (add_list, i);
	write (tempfd, lfnam, strlen (lfnam));
	write (tempfd, &cr, 1);
  }
  close (tempfd);
  
  switch (compression)
  {
      case 0:
	temptar = g_strdup (filename);
	totnum = num;
	break;
      case 1:
	command = g_strdup_printf ("%s -c %s", config.progs.gunzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c %s", config.progs.bunzip2, filename);
	break;
      default:
         FATAL_ERROR ("gz_tar_add:", _("Wrong Compression Parameter"))
  }
  if (compression)
  {
	temptar = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
	if ((tarfd = mkstemp (temptar)) == -1) 
	{
		FATAL_ERROR ("gz_tar_add:", _("Unable to create temporary file"))
		return -1;
	}
	DEBUG ("gz_tar_add: temp tar:", temptar);

	DEBUG ("tar temp command:", command);
	input = popen (command, "r");
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input)) FATAL_ERROR ("gz_tar_add:",strerror (ferror (input)))
		/* while (gtk_events_pending ()) gtk_main_iteration (); */ /* need it ?*/
		write (tarfd, entrystr, count);
	}
	totnum = num +1;
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
	g_free (entrystr);
	g_free (command);
	pclose (input);
	close (tarfd);
  }
  DEBUG ("gz_tar_add: Adding files to", temptar);
  command = g_strdup_printf ("%s rvvf %s -C %s -P -T %s", config.progs.tar, temptar, add_dir,  tempfile);
  DEBUG ("tar add command:", command);
  input = popen (command, "r");
  entrystr = g_malloc (8192);
  while (TRUE)
  {
      fgets (entrystr, 8192, input);
      /* Read file names and put them in the main clist */
	g_strchomp (entrystr);
	entry = gz_tar_entry (entrystr);
	gtk_clist_append ((GtkCList *)main_clist, (gchar **)entry);
      if (ferror (input)) FATAL_ERROR ("gz_tar_add:",strerror (ferror (input)))
          /* Check if the line is not too long */
      if (feof (input)) break;
      DEBUG ("gz_tar_add:", entrystr);
      /* update prograss bar */
      g_strchomp (entrystr);
      if (entrystr[strlen (entrystr) - 1] != '/')
      {
	  if (innum == totnum) innum = 0; /* I can't know how many files there are */
          gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
          while (gtk_events_pending ()) gtk_main_iteration ();
      }

  }
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) 1); /* fill bar */
  g_free (entrystr);
  g_free (command);
  pclose (input);
  unlink (tempfile);
  g_free (tempfile);
  switch (compression)
  {
      case 0:
	break;
      case 1:
	command = g_strdup_printf ("%s -c > %s", config.progs.gzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c > %s", config.progs.bzip2, filename);
	break;
      default:
	/* should never happen */
        FATAL_ERROR ("gz_tar_add:", _("Internal Error - Wrong Compression Parameter(2)"))
  }
  if (compression)
  {
	DEBUG ("gzip file", temptar);
	DEBUG ("tar untemp command:", command);
	if (!(output = popen (command, "w")))
	{
		ERROR ("gz_tar_add", "Unable to launch gzip")
		return -1;
	}
	stat (temptar, &filestat);
	totnum = filestat.st_size;
	printf ("Total file size %d\n", totnum);
	innum = 0;
	if ((input = fopen (temptar, "r")) == NULL)
	{
		ERROR ("gz_tar_add", strerror (ferror (output)))
		return -1;
	}	
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input))
			FATAL_ERROR ("gz_tar_add:", strerror (errno))
		innum += count;
		fwrite (entrystr, 1, count, output);
		if (ferror (output))
			FATAL_ERROR ("gz_tar_add:", strerror (ferror (output)))
		gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat)innum/totnum);
		while (gtk_events_pending ()) gtk_main_iteration ();
	}
	g_free (entrystr);
	g_free (command);
	pclose (output);
	fclose (input);
	unlink (temptar);
  }
  g_free (temptar);
  return 0;
}
