/*
 * zip.c
 * This file provides the zip file operations interface
 */

#include "common.h"
#include "zip.h"

extern gz_config config;

GList *
gz_zip_list (const gchar *filename)
{
  GList *list = NULL;
  char  *command;
  FILE  *input = NULL;
  gz_list_entry *entry;

  command = g_strdup_printf ("%s -l -qq \"%s\"", config.progs.unzip, filename);
  DEBUG ("zip command:", command)
  input = popen (command, "r");
  if (input == NULL) ERROR ("gz_zip_list", _("cannot execute zip command"))

  while (TRUE)
  {
    char *size, *date, *time, *name, *path;
    char entrystr [255 +PATH_MAX +NAME_MAX]; /* No more than 255 size/date/... chars */
    char *pes; /* entry string pointer */
    int len; /* string lenght */

    /* Read a line at a time */
    entrystr[0] = '\0';
    fgets (entrystr, 255 +PATH_MAX +NAME_MAX, input);
    if (ferror (input)) FATAL_ERROR ("gz_zip_list:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) break;
    if (entrystr[strlen (entrystr) - 1] != '\n')
        WARNING ("gz_zip_list:", _("Too many chars in a file entry"))
    entrystr[strlen (entrystr) - 1] = '\0';
    DEBUG ("gz_zip_list:", entrystr)

    g_strchomp (entrystr);

    if (entrystr[strlen (entrystr) - 1] != '/') /* do not show stored path as files */
    {

        entry = g_malloc0 (sizeof(gz_list_entry));

        entry->perms = NULL; /* info not available */
        entry->owner = NULL; /* info not available */

        pes = entrystr;

        g_strchug (pes); /* Get Out spaces */
        size = strsep(&pes, " ");
        if ((len = strlen (size)) > 10)
        {
          WARNING ("gz_zip_list:", _("size field too long"))
          len = 10;
        }
        entry->size = g_malloc0 (len+1);
        strncpy (entry->size, size, len);
/*    DEBUG ("gz_zip_list:", entry->size); */
     
        g_strchug (pes); /* Get Out spaces */
        date = strsep(&pes, " ");
        if ((len = strlen (date)) > 8)
        {
          WARNING ("gz_zip_list:", _("date field too long"))
          len = 8;
        }
        entry->date = g_malloc0 (len+1);
        strncpy (entry->date, date, len);
/*    DEBUG ("gz_zip_list:", entry->date); */
	 
        g_strchug (pes); /* Get Out spaces */
        time = strsep(&pes, " ");
        if ((len = strlen (time)) > 5)
        {
          WARNING ("gz_zip_list:", _("time field too long"))
          len = 5;
        }
        entry->time = g_malloc0 (len+1);
        strncpy (entry->time, time, len);
/*    DEBUG ("gz_zip_list:", entry->time); */
    
        g_strchug (pes); /* Get Out spaces */
	path = pes;
	if ((name = strrchr (path, '/')))
	{
	    name++;
	    
	    if ((len = (strlen (path) - strlen (name))) > PATH_MAX)
	    {
		 WARNING ("gz_zip_list:", _("path name too long"))
		 len = PATH_MAX;
	    }
	    entry->path = g_malloc0 (len+1);
	    strncpy (entry->path, path, len);
/*    DEBUG ("gz_zip_list:", entry->path); */
	}
	else name = pes;
	if ((len = strlen (name)) > NAME_MAX) 
        {
          WARNING ("gz_zip_list:", _("name field too long"))
          len = NAME_MAX;
        }	
        entry->name = g_malloc0 (len+1);
        strncpy (entry->name, name, len);
/*    DEBUG ("gz_zip_list:", entry->name); */

        list = g_list_append (list, entry);
    
    }
    
  }

  pclose (input);
  g_free (command);

  return list;

}

int
gz_zip_extract (gchar *filename, gchar *extr_dir)
{
  char *command;
  char *errstr;
  FILE  *input = NULL;
  gchar *barwname = "appbar1";
  gint innum = -1; /* unzip put an extra garbage line on stdout */
  GtkWidget *mainappbar;
  
  mainappbar = lookup_widget (main_win, barwname);

  errstr = g_strdup_printf ("Extract %s to: %s\n", filename, extr_dir);
  DEBUG ("gz_zip_extract:", errstr);
  g_free (errstr);
  command = g_strdup_printf ("%s -o \"%s\" -d \"%s\"", config.progs.unzip, filename, extr_dir);
  DEBUG ("zip command:", command);
  input = popen (command, "r");
  while (TRUE)
  {
    char entrystr [8192];

    fgets (entrystr, 8192, input);
    if (ferror (input)) FATAL_ERROR ("gz_zip_extract:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) break;
    DEBUG ("gz_zip_extract:", entrystr);
    /* update prograss bar */
    gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/flnum);
    while (gtk_events_pending ())
                gtk_main_iteration ();

  }
  pclose (input);
  return FALSE;
}
